from dataclasses import dataclass

from pathlib import Path
from random import choice, choices

from rich import print
from rich.prompt import Confirm, IntPrompt, Prompt

from .console import console
from .constants import (
    ABC_PARAMS,
    IRON_CARDIO_PARAMS,
    SUGGESTION,
    WARNING,
)
from .database import read_database


@dataclass
class Workout:
    bells: str
    variation: str
    time: int
    load: int
    units: str
    swings: int
    sets: int
    reps: int
    workout_type: str

    def display_workout(self) -> None:
        """Print an a workout to the console.
        :returns: None.
        """
        if self.swings:
            swings = f"   Swings: {self.swings} reps"
        else:
            swings = ""
        print(
            f"""{self.workout_type.upper()}\n[green]===================[/green]
    Bells: {self.bells.title()}
Variation: {self.variation}
     Time: {self.time} mins
     Load: {self.load} {self.units}
{swings}
"""
        )


def get_workout_params(workout_type: str) -> dict:
    match workout_type:
        case "iron-cardio" | "iron cardio":
            return "iron cardio", IRON_CARDIO_PARAMS
        case "abc" | "Armor Building Complex":
            return "Armor Building Complex", ABC_PARAMS


def random_workout(db_path: Path, workout_type: str) -> Workout:
    """Create a random workout based on workout_type.
    :param db_path: The Path to the database.
    :returns: A Session object with randomly generated parameters.
    """
    data = read_database(db_path)
    loads = data["ic_loads"]
    workout_type, workout_params = get_workout_params(workout_type)
    bells = choices(
        population=tuple(workout_params["bells"].keys()),
        weights=tuple(workout_params["bells"].values()),
    )[0]
    if bells == "Double Bells":
        variation = choices(
            population=tuple(workout_params["doublebell variations"].keys()),
            weights=tuple(workout_params["doublebell variations"].values()),
        )[0]
    elif bells == "Single Bell":
        variation = choices(
            population=tuple(workout_params["singlebell variations"].keys()),
            weights=tuple(workout_params["singlebell variations"].values()),
        )[0]
    time = choices(
        population=tuple(workout_params["times"].keys()),
        weights=tuple(workout_params["times"].values()),
    )[0]
    load = choices(
        population=tuple(workout_params["loads"].keys()),
        weights=tuple(workout_params["loads"].values()),
    )[0]
    swings = choices(
        population=tuple(workout_params["swings"].keys()),
        weights=tuple(workout_params["swings"].values()),
    )[0]
    load = loads[load]
    units = loads["units"]
    if swings:
        swings = choice(range(50, 160, 10))
    else:
        swings = 0
    return Workout(
        bells=bells,
        variation=variation,
        time=time,
        load=load,
        units=units,
        swings=swings,
        sets=0,
        reps=workout_params["rep schemes"][variation],
        workout_type=workout_type,
    )


def _get_options(session_param: dict) -> str:
    """Select options for a given Session parameter.
    :param session_param: The dictionary containing options for a Session parameter.
    :returns: A string consisting of the Session parameter choosen by the user.
    """
    options = list(session_param.keys())
    for i, option in enumerate(options, 1):
        print(f"    [{i}] {option}")
    while True:
        try:
            selection = IntPrompt.ask("Choose your option")
            return options[selection - 1]
        except IndexError:
            print(":warning: Not a valid option.", style=WARNING)
            print("Enter a number between 1 and {max(options) + 1}.", style=SUGGESTION)
            continue


def create_custom_workout(workout_type: str) -> Workout:
    """Create a custom Iron Cardio session.
    :returns: An Workout object created by the user.
    """
    workout_type, workout_params = get_workout_params(workout_type)
    bells = _get_options(workout_params["bells"])
    if bells == "Double Bells":
        variation = _get_options(workout_params["doublebell variations"])
    elif bells == "Single Bell":
        variation = _get_options(workout_params["singlebell variations"])
    time = IntPrompt.ask("How long was your session (in minutes)")
    units = _get_units()
    load = IntPrompt.ask(f"What weight did you use (in {units})")
    if Confirm.ask("Did you swing"):
        swings = IntPrompt.ask("How many swings")
    else:
        swings = 0
    return Workout(
        bells,
        variation,
        time,
        load,
        units,
        swings,
        0,
        workout_params["rep schemes"][variation],
        workout_type,
    )


def _get_units() -> str:
    """A helper function to get the units.
    :returns: A string, either 'pounds' or 'kilograms'.
    """
    while True:
        units = Prompt.ask("[P]ounds or [K]ilograms").lower()
        if units.startswith("p"):
            units = "pounds"
        elif units.startswith("k"):
            units = "kilograms"
        else:
            console.print(":warning: Invalid option", style=WARNING)
            console.print("Please enter P or K", style=SUGGESTION)
            continue
        break
    return units


def set_loads() -> dict:
    """Creates a dictionary containing the units and kettlebell weights and body weight
    of the user.
    :returns: A dict of the loads set by the user.
    """
    while True:
        units = _get_units()
        bodyweight = IntPrompt.ask("Bodyweight")
        console.print("Enter the weight for the...")
        light_load = IntPrompt.ask("Light kettlebell")
        medium_load = IntPrompt.ask("Medium kettlebell")
        heavy_load = IntPrompt.ask("Heavy kettlebell")
        loads = {
            "units": units,
            "bodyweight": bodyweight,
            "light load": light_load,
            "medium load": medium_load,
            "heavy load": heavy_load,
        }
        console.clear()
        for label, value in loads.items():
            console.print(f"{label.title()}: {value}")
        if Confirm.ask(
            "Are these loads correct? If you confirm, they will be used to generate Iron Cardio sessions."
        ):
            break
    return loads



from statistics import mean

import plotext as plt
from rich.table import Table

from .console import console
from .workouts import Workout, get_workout_params


def calc_session_stats(workout: Workout, bodyweight: int) -> dict:
    """Calculate the stats for a given session.
    :param session: The workout for which to calculate the stats.
    :param bodyweight: The user's bodyweight at time of the session.
    :returns: A dict containing total weight moved, number of reps, and the pace.
    """
    total_reps = workout.reps * workout.sets
    workout_params = get_workout_params(workout.workout_type)[1]

    if workout.bells == "Double Bells":
        load_factor = 2
    else:
        load_factor = 1

    if "Pullup" in workout.variation and workout.bells == "Double Bells":
        pullup_factor = 1
    elif "Pullup" in workout.variation and workout.bells == "Single Bell":
        pullup_factor = 0.5
    else:
        pullup_factor = 0

    stats = {
        "weight moved": (
            workout_params["rep schemes"][workout.variation] * workout.load * load_factor * workout.sets
            + (workout.swings * workout.load)
            + (bodyweight * int(workout.sets * pullup_factor))
        ),
        "reps": total_reps + int(workout.sets * pullup_factor),
        "pace": (workout.time * 60) / (total_reps + (workout.sets * pullup_factor)),
    }
    return stats


def display_session_stats(session: Workout, bodyweight: int) -> None:
    """Prints the stats for a given session.
    :param session: The Session object for which to display the stats.
    :param bodyweight: The bodyweight of the user.
    :returns: None"""
    stats = calc_session_stats(session, bodyweight)
    console.print(
        f"""Session Stats
[green]=============[/green]
Weight Moved: {stats.get("weight moved"):,} {session.units}
  Total Reps: {stats.get("reps")}
        Pace: {round(stats.get("pace"), 1)} sec/rep
    """
    )


def get_all_time_stats(data: dict) -> tuple[list[str], list[int]]:
    """Print stats from all workout in the database.
    :data: A dict of the data in the database.
    :returns: Lists of both dates and weight moved per session.
    """
    bodyweight = data["ic_loads"]["bodyweight"]
    units = data["ic_loads"]["units"]
    dates = []
    stats = []
    sessions = []
    for session_data in data["saved_sessions"]:
        date = session_data["date"]
        session = Workout(**session_data["session"])
        dates.append(date)
        stats.append(calc_session_stats(session, bodyweight))
        sessions.append(session)
    total_mins = sum(session.time for session in sessions)
    weight_per_session = [stat["weight moved"] for stat in stats]
    total_weight_moved = sum(weight_per_session)
    total_reps = sum(stat["reps"] for stat in stats)
    average_pace = mean(stat["pace"] for stat in stats)
    print("\nAll Time Stats")
    console.print("==============", style="green")
    print(f"    Total Sessions: {len(stats):,}")
    print(f"        Total Time: {total_mins} mins")
    print(f"Total Weight Moved: {total_weight_moved:,} {units}")
    print(f"        Total Reps: {total_reps:,}")
    print(f"      Average Pace: {average_pace:.1f} sec/rep")
    return dates, weight_per_session


def plot_sessions(dates: list[str], weight_per_session: list[int]) -> None:
    """
    :param dates: A list of the dates stored as a string.
    :param weight_per_session: A list of the weight moved per session.
    :returns: None
    """
    """Plot weight per session."""
    background_color = "yellow"
    foreground_color = "black"
    plt.clear_color()
    plt.date_form("Y-m-d")
    plt.plot(dates, weight_per_session, marker="hd", color=foreground_color)
    plt.ticks_color(foreground_color)
    plt.plotsize(100, 30)
    plt.canvas_color(background_color)
    plt.axes_color(background_color)
    plt.title("Weight Moved Per Session")
    plt.xlabel("Date")
    plt.show()


def get_best_sessions(data: dict):
    """Get the best sessions based on weight moved.
    :param data: A dict of the data from the database.
    :returns: None"""
    bodyweight = data["ic_loads"]["bodyweight"]
    units = data["ic_loads"]["units"]
    if units.startswith("k"):
        units = "kg"
    sessions = []
    for session_data in data["saved_sessions"]:
        date = session_data["date"]
        session = Workout(**session_data["session"])
        sessions.append((date, session, calc_session_stats(session, bodyweight)))

    best_sessions_weight = sorted(
        sessions, key=lambda x: x[2]["weight moved"], reverse=True
    )
    if len(best_sessions_weight) > 10:
        best_sessions_weight = best_sessions_weight[:10]

    columns = [
        ("Date", "green"),
        ("Variation", "magenta"),
        ("Time (mins)", "magenta"),
        (f"Weight Moved ({units})", "blue"),
        ("Reps", "blue"),
        ("Pace (sec/rep)", "blue"),
    ]
    weight_table = Table(title="Best Sessions by Weight moved")
    for col, style in columns:
        weight_table.add_column(col, style=style, justify="right")
    for date, session, stats in best_sessions_weight:
        weight_table.add_row(
            date,
            session.variation,
            f"{session.time}",
            f"{stats['weight moved']:,}",
            f"{stats['reps']}",
            f"{stats['pace']:.1f}",
        )
    console.print(weight_table)



"""constants for iron cardio session parameters."""

from pathlib import Path

WARNING = "red"
SUGGESTION = "yellow"

KETTLEBELLS_HOME = Path().home() / ".kettlebells"
KETTLEBELLS_DB = KETTLEBELLS_HOME / "kettlebells_db.json"

DATE_FORMAT = "%Y-%m-%d"


IRON_CARDIO_PARAMS = {
    "bells": {"Single Bell": 4 / 6, "Double Bells": 2 / 6},
    "doublebell variations": {
        "Double Classic": 3 / 6,
        "Double Traveling 2s": 1 / 6,
        "Double Classic + Pullup": 1 / 6,
    },
    "singlebell variations": {
        "Classic": 3 / 10,
        "Classic + Pullup": 1 / 10,
        "Classic + Snatch": 1 / 10,
        "Traveling 2s": 1 / 10,
        "Traveling 2s + Snatch": 1 / 10,
        "Traveling 2s + Pullup": 1 / 10,
    },
    "times": {
        30: 1 / 6,
        20: 4 / 6,
        10: 1 / 6,
    },
    "loads": {
        "heavy load": 2 / 6,
        "medium load": 3 / 6,
        "light load": 1 / 6,
    },
    "swings": {True: 2 / 6, False: 4 / 6},
    "rep schemes": {
        "Classic": 3,
        "Classic + Pullup": 3,
        "Double Classic": 3,
        "Double Classic + Pullup": 3,
        "Classic + Snatch": 4,
        "Traveling 2s": 4,
        "Traveling 2s + Pullup": 4,
        "Double Traveling 2s": 4,
        "Traveling 2s + Snatch": 5,
    },
}

ABC_PARAMS = {
    "bells": {"Single Bell": 2 / 6, "Double Bells": 4 / 6},
    "doublebell variations": {"Armor Building Complex": 1},
    "singlebell variations": {"Armor Building Complex 2.0": 1},
    "times": {
        25: 1 / 6,
        20: 2 / 6,
        15: 2 / 6,
        10: 1 / 6,
    },
    "loads": {
        "heavy load": 2 / 6,
        "medium load": 3 / 6,
        "light load": 1 / 6,
    },
    "swings": {True: 1 / 6, False: 5 / 6},
    "rep schemes": {
        "Armor Building Complex 2.0": 6,
        "Armor Building Complex": 6,
    },
}
